# -*- coding: utf-8 -*-
"""
The print proxy is a mechanism to provide fully variable pdf print creation process inside of the oereb
server with minimal impact on the behavior of it. The idea is to bind an external web accessible print
service to the oereb server. The extract is generated by the oereb server the result is sent to the print
instance and the created pdf is received and returned to the client. This is a proxy/passtrough mechanism.
"""
